# coding: utf-8

"""
    FlashBlade REST API

    A lightweight client for FlashBlade REST API 2.17, developed by Pure Storage, Inc. (http://www.purestorage.com/).

    The version of the OpenAPI document: 2.17
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json
from typing import Set, Dict, Any

from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr
from pypureclient.flashblade.FB_2_17.models.fixed_reference import FixedReference
from pypureclient.flashblade.FB_2_17.models.reference import Reference


class FileSystemExport(BaseModel):
    """
    FileSystemExport
    """
    id: Optional[StrictStr] = Field(default=None, description="A non-modifiable, globally unique ID chosen by the system.")
    name: Optional[StrictStr] = Field(default=None, description="Name of the object (e.g., a file system or snapshot).")
    context: Optional[Reference] = Field(default=None, description="The context in which the operation was performed. Valid values include a reference to any array which is a member of the same fleet. If the array is not a member of a fleet, `context` will always implicitly be set to the array that received the request. Other parameters provided with the request, such as names of volumes or snapshots, are resolved relative to the provided `context`.")
    enabled: Optional[StrictBool] = Field(default=None, description="Returns a value of `true` if and only if the export policy that manages this export is enabled AND the protocol is enabled on the file system.")
    export_name: Optional[StrictStr] = Field(default=None, description="The name of the export used by clients to mount the file system. Export names must be unique within the same protocol/server combination.")
    member: Optional[FixedReference] = Field(default=None, description="Reference to the file system the policy is applied to.")
    policy: Optional[Reference] = Field(default=None, description="Reference to the NFS export policy or SMB client policy.")
    policy_type: Optional[StrictStr] = Field(default=None, description="Policy type attached to the export. It is either SMB or NFS.")
    server: Optional[FixedReference] = Field(default=None, description="Reference to the server the export will be visible on.")
    share_policy: Optional[Reference] = Field(default=None, description="Reference to the SMB share policy (only used for SMB).")
    status: Optional[StrictStr] = Field(default=None, description="Explains why `enabled` is `false`. Valid values include `policy_disabled` and `protocol_disabled`.")
    __properties = ["id", "name", "context", "enabled", "export_name", "member", "policy", "policy_type", "server", "share_policy", "status"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.to_dict(include_readonly=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.as_request_dict())

    def as_request_dict(self) -> Dict[str, Any]:
        return self.to_dict(include_readonly=False)

    def to_dict(self, include_readonly: bool=True) -> Dict[str, Any]:

        """Returns the dictionary representation of the model using alias"""
        excluded_fields: Set[str] = set()
        if not include_readonly:
            excluded_fields.update([
                "id",
                "name",
                "context",
                "enabled",
                "policy_type",
                "status",
            ])
        none_fields: Set[str] = set()
        for _field in self.__fields__.keys():
            if super().__getattribute__(_field) is None:
                none_fields.add(_field)

        _dict = self.dict(by_alias=True, exclude=excluded_fields, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of context
        if _include_in_dict('context', include_readonly, excluded_fields, none_fields):
            _dict['context'] = self.context.to_dict(include_readonly=include_readonly)
        # override the default output from pydantic by calling `to_dict()` of member
        if _include_in_dict('member', include_readonly, excluded_fields, none_fields):
            _dict['member'] = self.member.to_dict(include_readonly=include_readonly)
        # override the default output from pydantic by calling `to_dict()` of policy
        if _include_in_dict('policy', include_readonly, excluded_fields, none_fields):
            _dict['policy'] = self.policy.to_dict(include_readonly=include_readonly)
        # override the default output from pydantic by calling `to_dict()` of server
        if _include_in_dict('server', include_readonly, excluded_fields, none_fields):
            _dict['server'] = self.server.to_dict(include_readonly=include_readonly)
        # override the default output from pydantic by calling `to_dict()` of share_policy
        if _include_in_dict('share_policy', include_readonly, excluded_fields, none_fields):
            _dict['share_policy'] = self.share_policy.to_dict(include_readonly=include_readonly)
        return _dict

    def __getitem__(self, key):
        return super().__getattribute__(key)

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def __delitem__(self, key):
        setattr(self, key, None)

    def __getattribute__(self, name: str) -> Any:
        _value = super().__getattribute__(name)
        if _value is None and name in self.__fields__.keys() and _should_raise_on_none():
            raise AttributeError
        return _value

    def __str__(self) -> str:
        return self.to_str()

    def __repr__(self) -> str:
        return self.to_str()

    @classmethod
    def from_json(cls, json_str: str) -> FileSystemExport:
        """Create an instance of FileSystemExport from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    @classmethod
    def from_dict(cls, obj: dict) -> FileSystemExport:
        """Create an instance of FileSystemExport from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FileSystemExport.parse_obj(obj)

        _obj = FileSystemExport.construct(_fields_set=None, **{
            "id": obj.get("id"),
            "name": obj.get("name"),
            "context": Reference.from_dict(obj.get("context")) if obj.get("context") is not None else None,
            "enabled": obj.get("enabled"),
            "export_name": obj.get("export_name"),
            "member": FixedReference.from_dict(obj.get("member")) if obj.get("member") is not None else None,
            "policy": Reference.from_dict(obj.get("policy")) if obj.get("policy") is not None else None,
            "policy_type": obj.get("policy_type"),
            "server": FixedReference.from_dict(obj.get("server")) if obj.get("server") is not None else None,
            "share_policy": Reference.from_dict(obj.get("share_policy")) if obj.get("share_policy") is not None else None,
            "status": obj.get("status")
        })
        return _obj

def _should_raise_on_none() -> bool:
    import importlib
    _package = importlib.import_module(__package__)
    return _package._attribute_error_on_none

def _include_in_dict(name: str, include_readonly: bool, excluded_fields: Set[str], none_fields: Set[str]) -> bool:
    if name in none_fields:
        return False
    return (include_readonly or name not in excluded_fields)

