# coding: utf-8

"""
    Pure1 Public REST API

    Pure1 Public REST API, developed by [Pure Storage, Inc.](https://www.purestorage.com)   The Pure1 REST API 2.0 offers one single form of authentication: OAuth 2.0 via the [Token Exchange protocol](https://datatracker.ietf.org/doc/draft-ietf-oauth-token-exchange).  OAuth 2.0 is an open protocol to allow secure authorization in a simple and standard method from web, mobile, desktop and background applications.  Note that the [Authentication](#section/Authentication) section below mentions 'API Key' as the security scheme type. This is solely for the purpose of allowing testing this API with [Swagger UI](https://static.pure1.purestorage.com/api-swagger/index.html).  [Knowledge base reference documentation](https://support.purestorage.com/Pure1/Pure1_Manage/Pure1_Manage_-_REST_API/Pure1_Manage_-_REST_API__Reference)

    The version of the OpenAPI document: 1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json
from typing import Set, Dict, Any

from typing import Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr
from pypureclient.pure1.Pure1_1_2.models.fixed_reference import FixedReference


class Subscription(BaseModel):
    """
    Subscription
    """
    as_of: Optional[StrictInt] = Field(default=None, alias="_as_of", description="The freshness of the data (timestamp in millis since epoch).")
    id: Optional[StrictStr] = Field(default=None, description="A non-modifiable, globally unique ID chosen by the system.")
    name: Optional[StrictStr] = Field(default=None, description="A non-modifiable, locally unique name chosen by the system.")
    expiration_date: Optional[StrictInt] = Field(default=None, description="Date when the subscription expires. Represented as a timestamp of 00:00 on that date in UTC, in milliseconds since UNIX epoch.")
    initial_name: Optional[StrictStr] = Field(default=None, description="A non-modifiable, locally unique name that the subscription was started with, never changes.")
    last_updated_date: Optional[StrictInt] = Field(default=None, description="The date of the last amendment to the current subscription. Represented as a timestamp of 00:00 on that date in UTC, in milliseconds since UNIX epoch.")
    organization: Optional[FixedReference] = Field(default=None, description="The Pure1 organization for the subscription.")
    partner_name: Optional[StrictStr] = Field(default=None, description="The partner who invoices for the subscription.")
    service: Optional[StrictStr] = Field(default=None, description="The service type of the subscription. Values include `Evergreen//One`, `Evergreen//Flex`, `FlashStack as a Service`, `Storage as a Service`, `PaaS (Block Storage Service)`, `PaaS (File and Object Storage Service)`, `PaaS (Data Protection Service)`, and `Pure1 Subscription`.")
    start_date: Optional[StrictInt] = Field(default=None, description="Date when the subscription starts. Represented as a timestamp of 00:00 on that date in UTC, in milliseconds since UNIX epoch.")
    status: Optional[StrictStr] = Field(default=None, description="Current status of the subscription. Values include `active`, `terminated`, `poc-expired`, and `signed`.")
    subscription_term: Optional[StrictInt] = Field(default=None, description="Length of the subscription in months.")
    __properties = ["_as_of", "id", "name", "expiration_date", "initial_name", "last_updated_date", "organization", "partner_name", "service", "start_date", "status", "subscription_term"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.to_dict(include_readonly=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.as_request_dict())

    def as_request_dict(self) -> Dict[str, Any]:
        return self.to_dict(include_readonly=False)

    def to_dict(self, include_readonly: bool=True) -> Dict[str, Any]:

        """Returns the dictionary representation of the model using alias"""
        excluded_fields: Set[str] = set()
        if not include_readonly:
            excluded_fields.update([
                "as_of",
                "id",
                "name",
                "initial_name",
                "partner_name",
            ])
        none_fields: Set[str] = set()
        for _field in self.__fields__.keys():
            if super().__getattribute__(_field) is None:
                none_fields.add(_field)

        _dict = self.dict(by_alias=True, exclude=excluded_fields, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of organization
        if _include_in_dict('organization', include_readonly, excluded_fields, none_fields):
            _dict['organization'] = self.organization.to_dict(include_readonly=include_readonly)
        return _dict

    def __getitem__(self, key):
        return super().__getattribute__(key)

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def __delitem__(self, key):
        setattr(self, key, None)

    def __getattribute__(self, name: str) -> Any:
        _value = super().__getattribute__(name)
        if _value is None and name in self.__fields__.keys() and _should_raise_on_none():
            raise AttributeError
        return _value

    def __str__(self) -> str:
        return self.to_str()

    def __repr__(self) -> str:
        return self.to_str()

    @classmethod
    def from_json(cls, json_str: str) -> Subscription:
        """Create an instance of Subscription from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    @classmethod
    def from_dict(cls, obj: dict) -> Subscription:
        """Create an instance of Subscription from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Subscription.parse_obj(obj)

        _obj = Subscription.construct(_fields_set=None, **{
            "as_of": obj.get("_as_of"),
            "id": obj.get("id"),
            "name": obj.get("name"),
            "expiration_date": obj.get("expiration_date"),
            "initial_name": obj.get("initial_name"),
            "last_updated_date": obj.get("last_updated_date"),
            "organization": FixedReference.from_dict(obj.get("organization")) if obj.get("organization") is not None else None,
            "partner_name": obj.get("partner_name"),
            "service": obj.get("service"),
            "start_date": obj.get("start_date"),
            "status": obj.get("status"),
            "subscription_term": obj.get("subscription_term")
        })
        return _obj

def _should_raise_on_none() -> bool:
    import importlib
    _package = importlib.import_module(__package__)
    return _package._attribute_error_on_none

def _include_in_dict(name: str, include_readonly: bool, excluded_fields: Set[str], none_fields: Set[str]) -> bool:
    if name in none_fields:
        return False
    return (include_readonly or name not in excluded_fields)

